#!/usr/bin/env python
#
# Copyright (C) 2012  W. Trevor King <wking@tremily.us>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Generate figures for the blog post.
"""

import numpy as _numpy

from crunch import *  # sloppy import for a one-off hack script.


good_piezo_file = '~/tmp/rsrch/calibrate-piezo/data/calibrate-piezo-2012-02-06T16-49-29.dat'

good_piezo_parameters = {
    'pits': [  # (z_bottom_bits, z_top_bits) pairs
        (32036, 32604),
        (32070, 32630),
        (32175, 32721),
        (32264, 32789),
        (32292, 32830),
        (32413, 32987),
        (32565, 33077),
        ],
    'pit_depth': 200e-9,  # meters
    'z_gain': 20,  # volts applied at piezo / volts output from DAC
    'output_gamma_z': 10./2**15  # volts output from DAC / bit
    }

interference_pull_file = '~/tmp/rsrch/calibrate-piezo/interference/2012-05-23T15-56-50.h5'
interference_pull_bounds = [20000, 35000, 33000, 35000]
interference_alpha_z = 1.56e10


if __name__ == '__main__':
    parameters,experiments = load_experiments()
    diff = _numpy.array([b-a for a,b in piezo_parameters['pits']], dtype=float)
    print('pit_depth = {:g} +/- {:g} bits'.format(diff.mean(), diff.std()))
    print('pit_depth = {:g} m'.format(piezo_parameters['pit_depth']))
    print_parameters(parameters, experiments)
    for experiment in experiments:
        if experiment.name == 'pull':
            for filename,kwargs in [
                ('pull-piezo.png', {
                        'xlabel': 'z piezo (m)',
                        'bounds': [-10e-9, 500e-9, -250e-12, 250e-12],
                        }),
                ('pull-protein.png', {
                        'xlabel': 'protein extension (m)',
                        'bounds': [-10e-9, 500e-9, -250e-12, 250e-12],
                        }),
                ('pull-contour-space.png', {
                        'xlabel': 'contour-space (m)',
                        'bounds': [140e-9, 350e-9, 0, 130e-12],
                        }),
                ]:
                figure = experiment.plot(**kwargs)
                figure.savefig(filename)
        else:
            experiment.plot(save=True)
    load_good_piezo = lambda: load_piezo(filename=good_piezo_file)
    piezo = Piezo(load_data=load_good_piezo)
    data = piezo.crunch(parameters=good_piezo_parameters)
    piezo.plot(filename='piezo-good.png')
    print('good piezo:')
    for k,v in data.items():
        print('  {} = {:g}'.format(k, v))

    data = load_pull(filename=interference_pull_file, with_temperature=False)
    figure = Pull._plot(
        z_piezo=_numpy.array(data['alpha_v'], dtype=float),
        z=_numpy.zeros(data['alpha_v'].shape),
        z_contour=_numpy.zeros(data['alpha_v'].shape),
        deflection=_numpy.array(data['alpha_f'], dtype=float),
        pull_temperature=297.15,  # only used for contour-space plots
        **pull_parameters)
    axes = figure.axes[0]
    axes.axis(interference_pull_bounds)
    axes.set_title('piezo calibration via interference')
    axes.set_xlabel('z piezo (bits)')
    axes.set_ylabel('deflection (bits)')
    figure.savefig('pull-interference.png')

    pull = experiments[-1]
    parameters['alpha_z'] = interference_alpha_z
    pull_parameters.update(parameters)
    pull.crunch(pull_parameters)
    pull.plot(
        filename='pull-contour-space-interference.png',
        xlabel='contour-space (m)',
        bounds=[140e-9, 350e-9, 0, 130e-12],
        )
